from typing import List

from django.utils.html import escape
from wagtail import hooks
from wagtail.documents.rich_text import DocumentLinkHandler


@hooks.register("register_rich_text_features", order=1)
def register_document_feature(features):
    features.register_link_type(CustomDocumentLinkHandler)


class CustomDocumentLinkHandler(DocumentLinkHandler):
    @classmethod
    def expand_db_attributes_many(cls, attrs_list: List[dict]) -> List[str]:
        return [
            '<a href="%s" target="_blank">' % escape(doc.url) if doc else "<a>"
            for doc in cls.get_many(attrs_list)
        ]



from django.utils.html import escape
from wagtail import hooks
from wagtail.rich_text import LinkHandler


@hooks.register("register_rich_text_features")
def register_core_features(features):
    features.default_features.append("blockquote")
    features.register_link_type(ExternalLinkHandler)
    features.register_link_type(EmailLinkHandler)


class ExternalLinkHandler(LinkHandler):
    identifier = "external"

    @classmethod
    def expand_db_attributes(cls, attrs):
        href = attrs["href"]
        return '<a href="%s" target="_blank" rel="noopener noreferrer">' % escape(href)


class EmailLinkHandler(LinkHandler):
    identifier = "email"

    @classmethod
    def expand_db_attributes(cls, attrs):
        href = attrs["href"]
        return '<a href="%s" class="notranslate">' % escape(href)



from django.utils.safestring import mark_safe
from wagtail.blocks import CharBlock, ChoiceBlock, StaticBlock, StructBlock

from baseapp_wagtail.base.blocks.basic_blocks.custom_image_chooser_block import (
    CustomImageChooserBlock,
)
from baseapp_wagtail.base.blocks.basic_blocks.custom_rich_text_block import (
    CustomRichTextBlock,
)

RICH_TEXT_FEATURES = ["bold", "italic", "link", "ul", "hr"]


class BannerBlock(StructBlock):
    title = CharBlock(required=True, use_json_field=True, max_length=50)
    description = CustomRichTextBlock(
        icon="pilcrow",
        required=False,
        features=RICH_TEXT_FEATURES,
        max_length=255,
    )

    hr = StaticBlock(
        admin_text=mark_safe("<hr />"),
        label=" ",
    )

    featured_image = CustomImageChooserBlock(label=" ", required=False)
    image_position = ChoiceBlock(
        choices=[
            ("left", "Left"),
            ("right", "Right"),
        ],
        blank=True,
        required=True,
        default="left",
        label="Image Position",
        help_text="This indicates the position of the image in the desktop view.",
    )

    def get_api_representation(self, value, context=None):
        serialized_data = super().get_api_representation(value, context)
        serialized_data.pop("hr")

        return serialized_data

    class Meta:
        template = "base/blocks/empty.html"
        icon = "image"



from .featured_image_stream_field import FeaturedImageStreamField  # noqa: F401
from .page_body_stream_field import (  # noqa: F401
    PageBodyStreamField,
    SectionStreamBlock,
)
from .standard_page_stream_block import StandardPageStreamBlock  # noqa: F401



from .basic_blocks.custom_image_block import CustomImageBlock  # noqa: F401
from .basic_blocks.custom_image_chooser_block import (  # noqa: F401
    CustomImageChooserBlock,
)
from .basic_blocks.custom_rich_text_block import CustomRichTextBlock  # noqa: F401
from .custom_blocks.banner_block import BannerBlock  # noqa: F401



from django.apps import AppConfig


class WagtailConfig(AppConfig):
    default = True
    name = "baseapp_wagtail"
    verbose_name = "BaseApp Wagtail"
    default_auto_field = "django.db.models.BigAutoField"



# Generated by Django 5.1.1 on 2024-11-18 17:03

import django.db.models.deletion
import taggit.managers
import wagtail.images.models
import wagtail.models.media
import wagtail.search.index
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("taggit", "0006_rename_taggeditem_content_type_object_id_taggit_tagg_content_8fc721_idx"),
        ("wagtailcore", "0094_alter_page_locale"),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomDocument",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=255, verbose_name="title")),
                ("file", models.FileField(upload_to="documents", verbose_name="file")),
                ("created_at", models.DateTimeField(auto_now_add=True, verbose_name="created at")),
                ("file_size", models.PositiveIntegerField(editable=False, null=True)),
                ("file_hash", models.CharField(blank=True, editable=False, max_length=40)),
                (
                    "collection",
                    models.ForeignKey(
                        default=wagtail.models.media.get_root_collection_id,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="wagtailcore.collection",
                        verbose_name="collection",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        blank=True,
                        help_text=None,
                        through="taggit.TaggedItem",
                        to="taggit.Tag",
                        verbose_name="tags",
                    ),
                ),
                (
                    "uploaded_by_user",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="uploaded by user",
                    ),
                ),
            ],
            options={
                "verbose_name": "document",
                "verbose_name_plural": "documents",
                "abstract": False,
            },
            bases=(wagtail.search.index.Indexed, models.Model),
        ),
        migrations.CreateModel(
            name="CustomImage",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("title", models.CharField(max_length=255, verbose_name="title")),
                (
                    "file",
                    wagtail.images.models.WagtailImageField(
                        height_field="height",
                        upload_to=wagtail.images.models.get_upload_to,
                        verbose_name="file",
                        width_field="width",
                    ),
                ),
                ("width", models.IntegerField(editable=False, verbose_name="width")),
                ("height", models.IntegerField(editable=False, verbose_name="height")),
                (
                    "created_at",
                    models.DateTimeField(
                        auto_now_add=True, db_index=True, verbose_name="created at"
                    ),
                ),
                ("focal_point_x", models.PositiveIntegerField(blank=True, null=True)),
                ("focal_point_y", models.PositiveIntegerField(blank=True, null=True)),
                ("focal_point_width", models.PositiveIntegerField(blank=True, null=True)),
                ("focal_point_height", models.PositiveIntegerField(blank=True, null=True)),
                ("file_size", models.PositiveIntegerField(editable=False, null=True)),
                (
                    "file_hash",
                    models.CharField(blank=True, db_index=True, editable=False, max_length=40),
                ),
                ("alt_text", models.TextField(blank=True)),
                (
                    "collection",
                    models.ForeignKey(
                        default=wagtail.models.media.get_root_collection_id,
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to="wagtailcore.collection",
                        verbose_name="collection",
                    ),
                ),
                (
                    "tags",
                    taggit.managers.TaggableManager(
                        blank=True,
                        help_text=None,
                        through="taggit.TaggedItem",
                        to="taggit.Tag",
                        verbose_name="tags",
                    ),
                ),
                (
                    "uploaded_by_user",
                    models.ForeignKey(
                        blank=True,
                        editable=False,
                        null=True,
                        on_delete=django.db.models.deletion.SET_NULL,
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="uploaded by user",
                    ),
                ),
            ],
            options={
                "abstract": False,
            },
            bases=(
                wagtail.images.models.ImageFileMixin,
                wagtail.search.index.Indexed,
                models.Model,
            ),
        ),
        migrations.CreateModel(
            name="CustomRendition",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True, primary_key=True, serialize=False, verbose_name="ID"
                    ),
                ),
                ("filter_spec", models.CharField(db_index=True, max_length=255)),
                (
                    "file",
                    wagtail.images.models.WagtailImageField(
                        height_field="height",
                        storage=wagtail.images.models.get_rendition_storage,
                        upload_to=wagtail.images.models.get_rendition_upload_to,
                        width_field="width",
                    ),
                ),
                ("width", models.IntegerField(editable=False)),
                ("height", models.IntegerField(editable=False)),
                (
                    "focal_point_key",
                    models.CharField(blank=True, default="", editable=False, max_length=16),
                ),
                (
                    "image",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="renditions",
                        to="baseapp_wagtail_medias.customimage",
                    ),
                ),
            ],
            options={
                "unique_together": {("image", "filter_spec", "focal_point_key")},
            },
            bases=(wagtail.images.models.ImageFileMixin, models.Model),
        ),
    ]



from django.utils.translation import gettext_lazy as _
from wagtail.images.formats import (
    Format,
    register_image_format,
    unregister_image_format,
)

unregister_image_format(
    Format("fullwidth", _("Full width"), "richtext-image full-width", "width-800")
)
register_image_format(
    Format("fullwidth", _("Full width"), "richtext-image full-width", "width-1472")
)
register_image_format(
    Format("originalsize", _("Original size"), "richtext-image original-size", "original")
)
register_image_format(
    Format(
        "centered",
        _("Centered"),
        "richtext-image centered",
        "width-300",
    )
)



from urllib.parse import urljoin

from baseapp_core.settings.env import env

WAGTAIL_INSTALLED_APPS = [
    # wagtail-headless-preview
    "wagtail_headless_preview",
    # wagtail
    "wagtail.api.v2",
    "wagtail.contrib.settings",
    "wagtail.contrib.redirects",
    "wagtail.embeds",
    "wagtail.sites",
    "wagtail.snippets",
    "wagtail.documents",
    "wagtail.images",
    "wagtail.search",
    "wagtail.admin",
    "wagtail.locales",
    "wagtail.users",
    "wagtail",
    # wagtail dependencies
    "modelcluster",
    "taggit",
]

WAGTAIL_INSTALLED_INTERNAL_APPS = [
    "baseapp_wagtail",
    "baseapp_wagtail.base",
    "baseapp_wagtail.medias",
]

WAGTAIL_MIDDLEWARE = [
    "wagtail.contrib.redirects.middleware.RedirectMiddleware",
]

# Locale settings
LANGUAGE_CODE = "en-us"
TIME_ZONE = "UTC"
USE_I18N = True
USE_L10N = True
USE_TZ = True

WAGTAIL_CONTENT_LANGUAGES = LANGUAGES = [
    ("en", "English"),
]

# Wagtail settings
WAGTAILADMIN_BASE_URL = env("URL", default="http://localhost:8000")

# Wagtail Preview
if "FRONT_URL" not in globals():
    FRONT_URL = env("FRONT_URL", "", required=False)

FRONT_HEADLESS_URL = urljoin(FRONT_URL, env("WAGTAIL_FRONT_URL_PATH", default="/pages"))
FRONT_PAGE_PREVIEW_URL = urljoin(
    FRONT_URL, env("WAGTAIL_FRONT_PAGE_PREVIEW_URL_PATH", default="/page-preview")
)
WAGTAIL_HEADLESS_PREVIEW = {
    "CLIENT_URLS": {"default": f"{FRONT_PAGE_PREVIEW_URL}"},
    "ENFORCE_TRAILING_SLASH": False,
}

WAGTAILIMAGES_IMAGE_MODEL = "baseapp_wagtail_medias.CustomImage"
WAGTAILDOCS_DOCUMENT_MODEL = "baseapp_wagtail_medias.CustomDocument"
WAGTAILDOCS_EXTENSIONS = ["csv", "docx", "key", "odt", "pdf", "pptx", "rtf", "txt", "xlsx", "zip"]

"""
These are the settings that must be defined in the project settings:
- WAGTAIL_SITE_NAME

And these are the settings that you must add to the .env file:
- WAGTAILADMIN_BASE_URL | env: URL
    -- URL of the Wagtail admin. This shouldn't have the path, unless there is a specific rule in
    the Nginx, for example.
    -- This is a recycled env var from the baseapp-core.
- FRONT_HEADLESS_URL | env: FRONT_URL + WAGTAIL_FRONT_URL_PATH
    -- URL PATH of the front-end application that will consume the API. The path of the URL must
    point to where the pages will be rendered.
    -- This is a new env var designed only for Wagtail.
- FRONT_PAGE_PREVIEW_URL | env: FRONT_URL + WAGTAIL_FRONT_PAGE_PREVIEW_URL_PATH
    -- URL PATH of the front-end application that will render the page previews.
    -- This is a new env var designed only for Wagtail.
"""



from django.http import Http404
from django.utils.encoding import uri_to_iri
from rest_framework.response import Response
from wagtail.contrib.redirects import models
from wagtail.contrib.redirects.api import RedirectsAPIViewSet
from wagtail.contrib.redirects.middleware import get_redirect
from wagtail.models import Site

from baseapp_wagtail.locale.utils import clear_pathname


class CustomRedirectsAPIViewSet(RedirectsAPIViewSet):
    body_fields = RedirectsAPIViewSet.body_fields + ["is_permanent"]
    html_path_queryset = None

    listing_default_fields = RedirectsAPIViewSet.listing_default_fields + [
        "is_permanent",
    ]

    def find_view(self, request):
        queryset = self.get_queryset()
        try:
            obj = self.find_object(queryset, request)
            if obj is None:
                raise self.model.DoesNotExist
        except (self.model.DoesNotExist, Http404):
            # Avoiding retrieving an error response so NextJs can cache the result
            return Response("not found")

        self.html_path_queryset = obj
        instance = self.get_object()
        serializer = self.get_serializer(instance)
        return Response(serializer.data)

    def find_object(self, queryset, request):
        if "html_path" in request.GET and (html_path := request.GET["html_path"]):
            redirect = self.get_redirect(
                request,
                html_path,
            )

            if not redirect:
                cleaned_html_path = clear_pathname(html_path)
                if cleaned_html_path != html_path:
                    redirect = self.get_redirect(
                        request,
                        cleaned_html_path,
                    )

            if redirect is None:
                raise Http404
            else:
                return redirect

        return super().find_object(queryset, request)

    def get_redirect(self, request, path):
        redirect = get_redirect(request, path)
        if redirect is None:
            redirect = self._get_redirect(request, path)
            if redirect is None:
                redirect = self._get_redirect(request, uri_to_iri(path))
        return redirect

    def _get_redirect(self, request, path):
        """
        If the default get_redirect fails, try to find it with a case insensitive query.
        """
        if "\0" in path:  # reject URLs with null characters, which crash on Postgres (#4496)
            return None

        site = Site.find_for_request(request)
        return models.Redirect.get_for_site(site).filter(old_path__iexact=path).first()

    def get_object(self):
        if self.html_path_queryset:
            return self.html_path_queryset
        return super().get_object()



from pathlib import Path

from baseapp_core.tests.settings import *  # noqa

from baseapp_wagtail.settings import *  # noqa
from baseapp_wagtail.settings import (
    WAGTAIL_INSTALLED_APPS,
    WAGTAIL_INSTALLED_INTERNAL_APPS,
    WAGTAIL_MIDDLEWARE,
)

ROOT_URLCONF = "testproject.urls"

BASE_DIR = Path(__file__).resolve().parent.parent

APPS_DIR = BASE_DIR

# Needed to handle images and documents. Each project will have it's own way of handling media files.
# Must be absolute URLs for use in emails.
MEDIA_ROOT = str(BASE_DIR.parent / "media")
MEDIA_URL = "/media/"
STATIC_ROOT = str(BASE_DIR.parent / "static")
STATIC_URL = "/static/"

if "INSTALLED_APPS" not in globals():
    INSTALLED_APPS = []

if "MIDDLEWARE" not in globals():
    MIDDLEWARE = []

INSTALLED_APPS += [
    # baseapp_wagtail
    "testproject.base",
    *WAGTAIL_INSTALLED_INTERNAL_APPS,
    *WAGTAIL_INSTALLED_APPS,
]

MIDDLEWARE += WAGTAIL_MIDDLEWARE

WAGTAIL_SITE_NAME = "Test Project"



from wagtail.blocks import CharBlock, StructBlock

from ..custom_image_chooser_block import CustomImageChooserBlock


class CustomImageBlock(StructBlock):
    def __init__(self, *args, **kwargs):
        image_sizes = kwargs.pop("image_sizes", None)
        required = kwargs.pop("required", False)
        local_blocks = [
            ("image", CustomImageChooserBlock(required=required, image_sizes=image_sizes)),
            (
                "alt_text",
                CharBlock(
                    required=False,
                    help_text='If this is a <a href="https://www.w3.org/WAI/tutorials/images/decorative/" target="_blank">decorative image</a>, please leave this field blank.',
                ),
            ),
        ]
        super().__init__(local_blocks, *args, **kwargs)

    def get_api_representation(self, value, context=None):
        serialized_data = super().get_api_representation(value, context)
        if not serialized_data["image"]:
            return None
        return serialized_data

    class Meta:
        icon = "image"
        template = "base/blocks/empty.html"



from wagtail.api.v2.serializers import PageSerializer as WagtailPageSerializer

from baseapp_wagtail.base.rest_framework.pages.serializers import PageUrlPath


class SitemapPageSerializer(WagtailPageSerializer):
    url_path = PageUrlPath(read_only=True)



from wagtail.blocks import StreamBlock
from wagtail.fields import StreamField

from baseapp_wagtail.base.blocks import CustomImageBlock


class FeaturedImageStreamField(StreamField):
    @staticmethod
    def create(*args, **kwargs):
        kwargs.setdefault("verbose_name", "Featured Image")
        kwargs.setdefault("null", True)
        kwargs.setdefault("blank", True)
        kwargs.setdefault("use_json_field", True)

        featured_image_stream_block = FeaturedImageStreamBlock(max_num=1)
        return FeaturedImageStreamField(featured_image_stream_block, **kwargs)


class FeaturedImageStreamBlock(StreamBlock):
    featured_image = CustomImageBlock()



from django.urls import include, path
from wagtail import urls as wagtail_urls
from wagtail.admin import urls as wagtailadmin_urls
from wagtail.documents import urls as wagtaildocs_urls

from .base.rest_framework.router import api_router as base_api_router
from .medias.rest_framework.router import api_router as medias_api_router

urlpatterns = [
    path("cms/", include(wagtailadmin_urls)),
    path("documents/", include(wagtaildocs_urls)),
    path("api/v2/base/", base_api_router.urls),
    path("api/v2/medias/", medias_api_router.urls),
    # TODO: Find a way to disable the Wagtail pages. If we remove this line, the system won't
    # generate the url_path. This path will render the wagtail pages using jinja2. Without it set,
    # the system won't provide a final path for the pages and we need it to generate the headless
    # pages.
    path("", include(wagtail_urls)),
]



from wagtail.blocks import CharBlock, StructBlock


class TestBlock(StructBlock):
    title = CharBlock(required=True, use_json_field=True, max_length=50)

    class Meta:
        template = "base/blocks/empty.html"
        icon = "placeholder"



from django.http import Http404
from django.urls import path
from rest_framework.response import Response
from wagtail.api.v2.views import PagesAPIViewSet
from wagtail.models import Locale

from .serializers import CustomPageSerializer


class CustomPagesAPIEndpoint(PagesAPIViewSet):
    base_serializer_class = CustomPageSerializer
    path_view_queryset = None

    meta_fields = PagesAPIViewSet.meta_fields + [
        "url_path",
        "ancestors",
    ]

    def path_view(self, request):
        queryset = self.get_queryset()
        try:
            obj = self.find_object(queryset, request)
            if obj is None:
                raise self.model.DoesNotExist
        except self.model.DoesNotExist:
            raise Http404("not found")

        self.path_view_queryset = self._maybe_get_translated_page(request, obj)

        instance = self.get_object()
        serializer = self.get_serializer(instance)
        return Response(serializer.data)

    def _maybe_get_translated_page(self, request, page):
        lang_code = request.GET.get("locale")
        if not lang_code:
            return page

        try:
            locale = Locale.objects.get(language_code=lang_code)
        except Locale.DoesNotExist:
            return page

        translation = page.get_translation_or_none(locale=locale)
        if not translation or not translation.live:
            return page

        return translation

    def get_object(self):
        if self.path_view_queryset:
            return self.path_view_queryset.specific
        return super().get_object()

    @classmethod
    def get_urlpatterns(cls):
        return [
            path("", cls.as_view({"get": "listing_view"}), name="listing"),
            path("<int:pk>/", cls.as_view({"get": "detail_view"}), name="detail"),
            path("find/", cls.as_view({"get": "find_view"}), name="find"),
            path("path/", cls.as_view({"get": "path_view"}), name="path"),
        ]



from wagtail.api.v2.router import WagtailAPIRouter

from .page_preview.views import PagePreviewAPIViewSet
from .pages.views import CustomPagesAPIEndpoint
from .redirects.views import CustomRedirectsAPIViewSet
from .sitemap.views import SitemapAPIViewSet

api_router = WagtailAPIRouter("baseappwagtailapi_base")

# Add the wagtail and custom endpoints using the "register_endpoint" method.
# The first parameter is the name of the endpoint (eg. pages, images). This is used in the URL of
# the endpoint.
# The second parameter is the endpoint class that handles the requests
api_router.register_endpoint("pages", CustomPagesAPIEndpoint)
api_router.register_endpoint("page_preview", PagePreviewAPIViewSet)
api_router.register_endpoint("sitemap/pages", SitemapAPIViewSet)
api_router.register_endpoint("redirects", CustomRedirectsAPIViewSet)

# If you need to register new endpoints, you can just override the path("api/v2/base/", base_api_router.urls)
# in the urls.py of your project.



from rest_framework.fields import Field
from wagtail.images.api.v2.serializers import ImageSerializer

DEFAULT_IMAGE_SIZES = {
    "small": "fill-320x320",
    "medium": "fill-768x432",
    "medium_square": "fill-768x768",
    "full": "original",
}


class ImageSizesField(Field):
    def __init__(self, image_sizes=None, **kwargs):
        self.image_sizes = dict(DEFAULT_IMAGE_SIZES)
        self.image_sizes.update(image_sizes or {})
        super().__init__(**kwargs)

    def get_attribute(self, instance):
        return instance

    def to_representation(self, value):
        if not value:
            return None

        image_sizes = {}
        for key, size in self.image_sizes.items():
            cropped_img = value.get_rendition(size)
            if cropped_img:
                image_sizes[key] = {
                    "width": cropped_img.width,
                    "height": cropped_img.height,
                    "image_url": cropped_img.full_url,
                }

        return image_sizes if image_sizes else None


class CustomImageSerializer(ImageSerializer):
    image_sizes = ImageSizesField(read_only=True)



from wagtail.blocks import RichTextBlock


class CustomRichTextBlock(RichTextBlock):
    def get_api_representation(self, value, context=None):
        if not value:
            return None
        return self.render(value, context=context)



from django import forms
from django.utils.functional import cached_property
from wagtail.blocks import StreamBlock
from wagtail.blocks.stream_block import StreamBlockAdapter
from wagtail.fields import StreamField
from wagtail.telepath import register


class PageBodyStreamField(StreamField):
    @staticmethod
    def create(*args, **kwargs):
        kwargs["verbose_name"] = "Page body"
        kwargs["blank"] = True
        kwargs["use_json_field"] = True
        section_stream_block = SectionStreamBlock(
            [
                ("section_stream_block", *args),
            ]
        )
        return PageBodyStreamField(section_stream_block, **kwargs)

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)


class SectionStreamBlock(StreamBlock):
    pass


class SectionBlockAdapter(StreamBlockAdapter):
    js_constructor = "baseapp_wagtail.stream_blocks.SectionStreamBlock"

    @cached_property
    def media(self):
        streamblock_media = super().media
        return forms.Media(
            js=streamblock_media._js + ["baseapp_wagtail/js/section-stream-block.js"],
            css=streamblock_media._css,
        )


register(SectionBlockAdapter(), SectionStreamBlock)



from baseapp_wagtail.base.models import BaseStandardPage
from baseapp_wagtail.base.stream_fields import PageBodyStreamField

from .stream_fields.standard_page_stream_block import StandardPageStreamBlock


class StandardPage(BaseStandardPage):
    body = PageBodyStreamField.create(
        StandardPageStreamBlock(required=False),
    )



from django.apps import apps
from django.contrib.auth.models import Permission
from django.core.management import call_command
from django.core.management.base import BaseCommand, CommandError
from django.db import transaction

from baseapp_wagtail.settings import (
    WAGTAIL_INSTALLED_APPS,
    WAGTAIL_INSTALLED_INTERNAL_APPS,
)


class Command(BaseCommand):
    help = "Reverts all migrations for the Wagtail package apps"

    def handle(self, *args, **options):
        if not options.get("no_input", False):
            self.stdout.write(
                self.style.WARNING(
                    "This will delete all Wagtail-related data. "
                    "Make sure you have backed up your database.\n"
                    "Are you sure you want to proceed? [y/N]"
                )
            )
            if input("").lower() != "y":
                self.stdout.write("Operation cancelled.")
                return

        for app in [*WAGTAIL_INSTALLED_INTERNAL_APPS, *WAGTAIL_INSTALLED_APPS]:
            try:
                with transaction.atomic():
                    app_label = self._get_app_label(app)
                    self._remove_permissions(app_label)
                    call_command("migrate", app_label, "zero")
                    self.stdout.write(
                        self.style.SUCCESS(f"Reverted migrations for {app} ({app_label})")
                    )
            except CommandError as e:
                self.stdout.write(self.style.ERROR(f"Failed to revert migrations for {app}: {e}"))

    def _get_app_label(self, app_name):
        all_apps = apps.get_app_configs()
        for app_config in all_apps:
            if app_config.name == app_name:
                return app_config.label
        raise CommandError(f"App with name {app_name} not found")

    def _remove_permissions(self, app_label):
        """
        Some apps use django.contrib.auth.models.Permission to manage permissions. To revert their
        migrations, we need to remove the permissions first.
        """
        try:
            Permission.objects.filter(content_type__app_label=app_label).delete()
            self.stdout.write(self.style.SUCCESS(f"Removed permissions for {app_label}"))
        except Exception as e:
            self.stdout.write(
                self.style.ERROR(f"Failed to remove permissions for {app_label}: {e}")
            )



from wagtail.api.v2.router import WagtailAPIRouter
from wagtail.images.api.v2.views import ImagesAPIViewSet

api_router = WagtailAPIRouter("baseappwagtailapi_medias")
api_router.register_endpoint("images", ImagesAPIViewSet)

# If you need to register new endpoints, you can just override the path("api/v2/medias/", medias_api_router.urls)
# in the urls.py of your project.



from django.urls import path
from django.utils.decorators import method_decorator
from django.views.decorators.cache import cache_page
from rest_framework.response import Response
from wagtail.api.v2.views import PagesAPIViewSet

from .serializers import SitemapPageSerializer


class SitemapAPIViewSet(PagesAPIViewSet):
    base_serializer_class = SitemapPageSerializer
    body_fields = [
        "id",
        "type",
        "title",
        "url_path",
        "locale",
        "last_published_at",
    ]
    meta_fields = []

    @method_decorator(cache_page(60 * 60 * 1))  # 1 hour
    def listing_view(self, request):
        queryset = self.get_queryset()
        self.check_query_parameters(queryset)
        queryset = self.filter_queryset(queryset)
        serializer = self.get_serializer(queryset, many=True)
        return Response(serializer.data)

    @classmethod
    def get_urlpatterns(cls):
        return [
            path("", cls.as_view({"get": "listing_view"}), name="listing"),
        ]



from collections import OrderedDict

from django.conf import settings
from django.db import models
from wagtail.documents.models import AbstractDocument, Document
from wagtail.images.models import AbstractImage, AbstractRendition, Image


class CustomImage(AbstractImage):
    alt_text = models.TextField(blank=True)

    admin_form_fields = Image.admin_form_fields + ("alt_text",)

    @property
    def default_alt_text(self):
        return self.alt_text


class CustomRendition(AbstractRendition):
    image = models.ForeignKey(CustomImage, on_delete=models.CASCADE, related_name="renditions")

    class Meta:
        unique_together = (("image", "filter_spec", "focal_point_key"),)

    @property
    def attrs_dict(self):
        """
        A dict of the src, width, height, and alt attributes for an <img> tag.
        This fix <img src="" /> generated by RichTextBlocks.
        """
        return OrderedDict(
            [
                ("src", self.full_url),
                ("width", self.width),
                ("height", self.height),
                ("alt", self.alt),
            ]
        )


class CustomDocument(AbstractDocument):
    admin_form_fields = Document.admin_form_fields

    @property
    def url(self):
        """
        Property based on image AbstractRendition.full_url property.
        """
        url = super().url
        if hasattr(settings, "WAGTAILADMIN_BASE_URL") and url.startswith("/"):
            url = settings.WAGTAILADMIN_BASE_URL + url
        return url



from wagtail.api.v2.serializers import get_serializer_class
from wagtail.images.blocks import ImageChooserBlock

from baseapp_wagtail.medias.serializers import CustomImageSerializer, ImageSizesField


class CustomImageChooserBlock(ImageChooserBlock):
    def __init__(self, *args, **kwargs):
        self.image_sizes = kwargs.pop("image_sizes", None)
        super().__init__(*args, **kwargs)

    def get_api_representation(self, value, context=None):
        if value:
            return self._serialize_image(value, context)
        else:
            return super().get_api_representation(value, context)

    def _serialize_image(self, image, context):
        serializer_class = get_serializer_class(
            image.__class__,
            ["id", "download_url", "image_sizes"],
            meta_fields=["*"],
            field_serializer_overrides=self._maybe_get_serializer_overrides(),
            base=CustomImageSerializer,
        )
        serializer = serializer_class(context=context)
        return serializer.to_representation(image)

    def _maybe_get_serializer_overrides(self):
        if self.image_sizes:
            return {"image_sizes": ImageSizesField(image_sizes=self.image_sizes, read_only=True)}
        return None



from rest_framework.fields import Field
from wagtail.api.v2.serializers import PageSerializer as WagtailPageSerializer
from wagtail.api.v2.serializers import get_serializer_class


class PageFeaturedImage(Field):
    def get_attribute(self, instance):
        obj = instance.specific
        if hasattr(obj, "featured_image"):
            return obj.featured_image
        return None

    def to_representation(self, value):
        if not value:
            return None
        item = list(value).pop()
        return item.block.get_api_representation(item.value, context=self.context)


class OptionalMetaField(Field):
    """
    Some fields set in the page model might not be present in other page models that use the same
    serializer. This class is used to handle the absence of the field in the model.
    """

    def get_attribute(self, instance):
        obj = instance.specific
        if hasattr(obj, self.field_name):
            return getattr(obj, self.field_name)
        return None

    def to_representation(self, value):
        if not value:
            return None
        return value


class PageUrlPath(Field):
    def get_attribute(self, instance):
        return instance

    def to_representation(self, page):
        try:
            url_parts = page.get_url_parts()
            _, _, page_path = url_parts
            return page_path
        except (AttributeError, ValueError):
            return None


class PageAncestors(Field):
    def get_attribute(self, instance):
        return instance

    def to_representation(self, page):
        """
        Method based on get_breadcrumbs_items_for_page from Wagtail.
        """
        include_self = False
        pages = (
            page.get_ancestors(inclusive=include_self)
            .descendant_of(page.get_first_root_node(), inclusive=False)
            .filter(live=True)
            .specific(defer=True)
        )

        ancestors = []
        for ancestor in pages:
            serializer_class = get_serializer_class(
                ancestor.__class__,
                ["id", "type", "title", "url_path", "locale"],
                meta_fields=["*"],
                base=CustomPageSerializer,
            )
            serializer = serializer_class(context=self.context)
            ancestors.append(serializer.to_representation(ancestor))
        return ancestors


class CustomPageSerializer(WagtailPageSerializer):
    featured_image = PageFeaturedImage(read_only=True)
    url_path = PageUrlPath(read_only=True)
    ancestors = PageAncestors(read_only=True)



from wagtail.blocks import StreamBlock

from ..blocks import BannerBlock, CustomRichTextBlock


class StandardPageStreamBlock(StreamBlock):
    rich_text_block = CustomRichTextBlock(icon="pilcrow")
    banner_block = BannerBlock()



import re

from wagtail.models import Locale


def clear_pathname(pathname: str) -> str:
    locales = Locale.objects.values_list("language_code", flat=True)
    locale_pattern = re.compile(r"^/({})(/|$)".format("|".join(locales)))
    return re.sub(locale_pattern, "/", pathname)



from urllib.parse import urlparse

from django.conf import settings
from django.utils.translation import gettext as _
from wagtail.admin.panels import FieldPanel
from wagtail.api import APIField
from wagtail.models import Page, PageBase
from wagtail.search import index
from wagtail_headless_preview.models import HeadlessPreviewMixin

from .stream_fields import (
    FeaturedImageStreamField,
    PageBodyStreamField,
    StandardPageStreamBlock,
)


class HeadlessPageBase(PageBase):
    def __init__(cls, *args, **kwargs):
        super().__init__(*args, **kwargs)
        # Guarantee that the jinja2 template will be empty.
        cls.template = "pages/empty.html"


class HeadlessPageMixin(HeadlessPreviewMixin):
    @property
    def headless_url(self, request=None, current_site=None):
        url = self.get_url(request, current_site)
        if (root_url := settings.FRONT_HEADLESS_URL) and url and self._has_no_domain(url):
            return root_url + url
        return url

    # This is a workaround to fix the headless_url property for "View live" buttons.
    url = headless_url

    def _has_no_domain(self, url: str) -> bool:
        parsed_url = urlparse(url)
        return not parsed_url.netloc

    class Meta:
        abstract = True


class DefaultPageModel(HeadlessPageMixin, Page, metaclass=HeadlessPageBase):
    featured_image = FeaturedImageStreamField.create()
    body = None

    content_panels = Page.content_panels + [
        FieldPanel("featured_image", classname="collapsed"),
        FieldPanel("body"),
    ]

    api_fields = [
        APIField("featured_image"),
        APIField("body"),
    ]

    search_fields = Page.search_fields + [
        index.SearchField("body"),
        index.AutocompleteField("body"),
    ]

    class Meta:
        abstract = True


class BaseStandardPage(DefaultPageModel):
    body = PageBodyStreamField.create(
        StandardPageStreamBlock(required=False),
    )

    class Meta:
        verbose_name = _("Standard page")
        verbose_name_plural = _("Standard pages")
        abstract = True



from .settings import *  # noqa

if "INSTALLED_APPS" not in globals():
    INSTALLED_APPS = []

INSTALLED_APPS += ["baseapp_wagtail.tests"]

# Constance
CONSTANCE_BACKEND = "constance.backends.memory.MemoryBackend"



# Generated by Django 5.1.1 on 2024-10-29 19:10

import django.db.models.deletion
from django.db import migrations, models

import baseapp_wagtail.base.models
import baseapp_wagtail.base.stream_fields.featured_image_stream_field
import baseapp_wagtail.base.stream_fields.page_body_stream_field


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("wagtailcore", "0094_alter_page_locale"),
    ]

    operations = [
        migrations.CreateModel(
            name="StandardPage",
            fields=[
                (
                    "page_ptr",
                    models.OneToOneField(
                        auto_created=True,
                        on_delete=django.db.models.deletion.CASCADE,
                        parent_link=True,
                        primary_key=True,
                        serialize=False,
                        to="wagtailcore.page",
                    ),
                ),
                (
                    "featured_image",
                    baseapp_wagtail.base.stream_fields.featured_image_stream_field.FeaturedImageStreamField(
                        [("featured_image", 2)],
                        blank=True,
                        block_lookup={
                            0: (
                                "baseapp_wagtail.base.blocks.basic_blocks.custom_image_chooser_block.block.CustomImageChooserBlock",
                                (),
                                {"image_sizes": None, "required": False},
                            ),
                            1: (
                                "wagtail.blocks.CharBlock",
                                (),
                                {
                                    "help_text": 'If this is a <a href="https://www.w3.org/WAI/tutorials/images/decorative/" target="_blank">decorative image</a>, please leave this field blank.',
                                    "required": False,
                                },
                            ),
                            2: (
                                "wagtail.blocks.StructBlock",
                                [[("image", 0), ("alt_text", 1)]],
                                {},
                            ),
                        },
                        null=True,
                        verbose_name="Featured Image",
                    ),
                ),
                (
                    "body",
                    baseapp_wagtail.base.stream_fields.page_body_stream_field.PageBodyStreamField(
                        [("section_stream_block", 8)],
                        blank=True,
                        block_lookup={
                            0: (
                                "baseapp_wagtail.base.blocks.basic_blocks.custom_rich_text_block.block.CustomRichTextBlock",
                                (),
                                {"icon": "pilcrow"},
                            ),
                            1: (
                                "wagtail.blocks.CharBlock",
                                (),
                                {"max_length": 50, "required": True, "use_json_field": True},
                            ),
                            2: (
                                "baseapp_wagtail.base.blocks.basic_blocks.custom_rich_text_block.block.CustomRichTextBlock",
                                (),
                                {
                                    "features": ["bold", "italic", "link", "ul", "hr"],
                                    "icon": "pilcrow",
                                    "max_length": 255,
                                    "required": False,
                                },
                            ),
                            3: (
                                "wagtail.blocks.static_block.StaticBlock",
                                (),
                                {"admin_text": "<hr />", "label": " "},
                            ),
                            4: (
                                "baseapp_wagtail.base.blocks.basic_blocks.custom_image_chooser_block.block.CustomImageChooserBlock",
                                (),
                                {"label": " ", "required": False},
                            ),
                            5: (
                                "wagtail.blocks.ChoiceBlock",
                                [],
                                {
                                    "blank": True,
                                    "choices": [("left", "Left"), ("right", "Right")],
                                    "help_text": "This indicates the position of the image in the desktop view.",
                                    "label": "Image Position",
                                },
                            ),
                            6: (
                                "wagtail.blocks.StructBlock",
                                [
                                    [
                                        ("title", 1),
                                        ("description", 2),
                                        ("hr", 3),
                                        ("featured_image", 4),
                                        ("image_position", 5),
                                    ]
                                ],
                                {},
                            ),
                            7: ("wagtail.blocks.StructBlock", [[("title", 1)]], {}),
                            8: (
                                "wagtail.blocks.StreamBlock",
                                [[("rich_text_block", 0), ("banner_block", 6), ("test_block", 7)]],
                                {"required": False},
                            ),
                        },
                        verbose_name="Page body",
                    ),
                ),
            ],
            options={
                "verbose_name": "Standard page",
                "verbose_name_plural": "Standard pages",
                "abstract": False,
            },
            bases=(baseapp_wagtail.base.models.HeadlessPageMixin, "wagtailcore.page"),
        ),
    ]
